@@ -33,6 +33,11 @@ static GObjectClass *parent_class = NULL;
 #define MAGIC_RADIO_GROUP_KEY "Bonobo::RadioGroupName"
 #define UI_SYNC_MENU_KEY "Bonobo::UISyncMenu"
 
+static GQuark menu_id = 0;
+static GQuark popups_id = 0;
+static GQuark submenu_id = 0;
+static GQuark menuitem_id = 0;
+
 typedef struct {
 	GtkMenu          *menu;
 	char             *path;
@@ -281,38 +286,6 @@ cmd_get_menu_pixbuf (GtkWidget    *widget,
 	return NULL;
 }
 
-/*
- * The second parameter uses underscores to escape keyboard
- * shortcuts. We ignore the first underscore, but if there
- * are two in a row, they are treated as if there was one.
- */
-static gboolean
-str_uscore_equal (const char *plain, const char *scored)
-{
-	while (*plain || *scored) {
-		if (*scored == '_') {
-			scored++;
-
-			/*
-			 * Must check for badly formed string
-			 * (ending with single underscore)
-			 * or risk reading off end.
-			 */
-			if (!*scored)
-				return FALSE;
-		}
-
-		if (*scored != *plain)
-			return FALSE;
-		else {
-			scored++;
-			plain++;
-		}
-	}
-
-	return TRUE;
-}
-
 static gboolean
 label_same (GtkBin *menu_widget, const char *txt)
 {
@@ -320,9 +293,9 @@ label_same (GtkBin *menu_widget, const char *txt)
 
 	return menu_widget &&
 		(label = menu_widget->child) &&
-		GTK_IS_ACCEL_LABEL (label) &&
+		GTK_IS_LABEL (label) &&
 		((GtkLabel *)label)->label &&
-		str_uscore_equal (((GtkLabel *)label)->label, txt);
+		!strcmp (((GtkLabel *)label)->label, txt);
 }
 
 static void
@@ -394,27 +367,32 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 	
 	if ((label_attr = bonobo_ui_engine_get_attr (node, cmd_node, "label"))) {
 		GtkWidget *label;
-
 		if (!label_same (GTK_BIN (menu_widget), label_attr)) {
-			label = gtk_accel_label_new (label_attr);
+			if (!GTK_BIN (menu_widget)->child) {
+				label = gtk_accel_label_new (label_attr);
 
-			/* this widget has a mnemonic */
-			gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+				g_object_freeze_notify (G_OBJECT (label));
+
+				/* this widget has a mnemonic */
+				gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+
+				/* Setup the widget. */
+				gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+				gtk_widget_show (label);
+				gtk_container_add (GTK_CONTAINER (menu_widget), label);
+
+			} else {
+				label = GTK_BIN (menu_widget)->child;
+
+				g_object_freeze_notify (G_OBJECT (label));
+
+				gtk_label_set_text_with_mnemonic (GTK_LABEL (label), label_attr);
+			}
 
-			/* Setup the widget. */
-			gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-			gtk_widget_show (label);
-			
-			/*
-			 * Insert it into the menu item widget and setup the
-			 * accelerator. FIXME: rather inefficient.
-			 */
-			if (GTK_BIN (menu_widget)->child)
-				gtk_widget_destroy (GTK_BIN (menu_widget)->child);
-			
-			gtk_container_add (GTK_CONTAINER (menu_widget), label);
 			gtk_accel_label_set_accel_widget (
 				GTK_ACCEL_LABEL (label), menu_widget);
+
+			g_object_thaw_notify (G_OBJECT (label));
 		} /* else
 			g_warning ("No change in label '%s'", label_attr); */
 
@@ -849,8 +827,17 @@ static gboolean
 impl_bonobo_ui_sync_menu_can_handle (BonoboUISync *sync,
 				     BonoboUINode *node)
 {
-	return bonobo_ui_node_has_name (node, "menu") ||
-	       bonobo_ui_node_has_name (node, "popups");
+	if (!menu_id) {
+		menu_id = g_quark_from_static_string ("menu");
+		popups_id = g_quark_from_static_string ("popups");
+		submenu_id = g_quark_from_static_string ("submenu");
+		menuitem_id = g_quark_from_static_string ("menuitem");
+	}
+	
+	return (node->name_id == menu_id ||
+		node->name_id == popups_id ||
+		node->name_id == submenu_id ||
+		node->name_id == menuitem_id);
 }
 
 /* We need to map the shell to the item */
